@@ -1,41 +1,27 @@
 #-*-Mode:perl;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 # ex: set ft=perl fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
-# BSD LICENSE
-# 
-# Copyright (c) 2014-2016, Michael Truog <mjtruog at gmail dot com>
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-# 
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in
-#       the documentation and/or other materials provided with the
-#       distribution.
-#     * All advertising materials mentioning features or use of this
-#       software must display the following acknowledgment:
-#         This product includes software developed by Michael Truog
-#     * The name of the author may not be used to endorse or promote
-#       products derived from this software without specific prior
-#       written permission
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-# DAMAGE.
+# MIT License
+#
+# Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the "Software"),
+# to deal in the Software without restriction, including without limitation
+# the rights to use, copy, modify, merge, publish, distribute, sublicense,
+# and/or sell copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
 #
 
 package CloudI::API;
@@ -43,7 +29,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '1.53';
+$CloudI::API::VERSION = '1.74';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -136,9 +122,6 @@ sub new
         _timeout_sync => undef,
         _timeout_terminate => 1000, # TIMEOUT_TERMINATE_MIN
         _priority_default => undef,
-        _request_timeout_adjustment => undef,
-        _request_timeout => undef,
-        _request_timer => undef,
     }, $class;
     $self->_send(Erlang::term_to_binary(Erlang::OtpErlangAtom->new('init')));
     ($self->{_process_index},
@@ -150,8 +133,7 @@ sub new
      $self->{_timeout_async},
      $self->{_timeout_sync},
      $self->{_timeout_terminate},
-     $self->{_priority_default},
-     $self->{_request_timeout_adjustment}) = $self->_poll_request(undef, 0);
+     $self->{_priority_default}) = $self->_poll_request(undef, 0);
     return $self;
 }
 
@@ -310,14 +292,14 @@ sub mcast_async
 sub forward_
 {
     my $self = shift;
-    my ($command, $name, $request_info, $request,
+    my ($request_type, $name, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    if ($command == ASYNC)
+    if ($request_type == ASYNC)
     {
         $self->forward_async($name, $request_info, $request,
                              $timeout, $priority, $trans_id, $pid);
     }
-    elsif ($command == SYNC)
+    elsif ($request_type == SYNC)
     {
         $self->forward_sync($name, $request_info, $request,
                             $timeout, $priority, $trans_id, $pid);
@@ -333,21 +315,6 @@ sub forward_async
     my $self = shift;
     my ($name, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    if ($self->{_request_timeout_adjustment})
-    {
-        if ($timeout == $self->{_request_timeout})
-        {
-            my $elapsed = _max(0, _milliseconds() - $self->{_request_timer});
-            if ($elapsed > $timeout)
-            {
-                $timeout = 0;
-            }
-            else
-            {
-                $timeout -= $elapsed;
-            }
-        }
-    }
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('forward_async'), $name,
         Erlang::OtpErlangBinary->new($request_info),
@@ -361,21 +328,6 @@ sub forward_sync
     my $self = shift;
     my ($name, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    if ($self->{_request_timeout_adjustment})
-    {
-        if ($timeout == $self->{_request_timeout})
-        {
-            my $elapsed = _max(0, _milliseconds() - $self->{_request_timer});
-            if ($elapsed > $timeout)
-            {
-                $timeout = 0;
-            }
-            else
-            {
-                $timeout -= $elapsed;
-            }
-        }
-    }
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('forward_sync'), $name,
         Erlang::OtpErlangBinary->new($request_info),
@@ -387,14 +339,14 @@ sub forward_sync
 sub return_
 {
     my $self = shift;
-    my ($command, $name, $pattern, $response_info, $response,
+    my ($request_type, $name, $pattern, $response_info, $response,
         $timeout, $trans_id, $pid) = @_;
-    if ($command == ASYNC)
+    if ($request_type == ASYNC)
     {
         $self->return_async($name, $pattern, $response_info, $response,
                             $timeout, $trans_id, $pid);
     }
-    elsif ($command == SYNC)
+    elsif ($request_type == SYNC)
     {
         $self->return_sync($name, $pattern, $response_info, $response,
                            $timeout, $trans_id, $pid);
@@ -410,23 +362,6 @@ sub return_async
     my $self = shift;
     my ($name, $pattern, $response_info, $response,
         $timeout, $trans_id, $pid) = @_;
-    if ($self->{_request_timeout_adjustment})
-    {
-        if ($timeout == $self->{_request_timeout})
-        {
-            my $elapsed = _max(0, _milliseconds() - $self->{_request_timer});
-            if ($elapsed > $timeout)
-            {
-                $response_info = '';
-                $response = '';
-                $timeout = 0;
-            }
-            else
-            {
-                $timeout -= $elapsed;
-            }
-        }
-    }
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('return_async'), $name, $pattern,
         Erlang::OtpErlangBinary->new($response_info),
@@ -440,23 +375,6 @@ sub return_sync
     my $self = shift;
     my ($name, $pattern, $response_info, $response,
         $timeout, $trans_id, $pid) = @_;
-    if ($self->{_request_timeout_adjustment})
-    {
-        if ($timeout == $self->{_request_timeout})
-        {
-            my $elapsed = _max(0, _milliseconds() - $self->{_request_timer});
-            if ($elapsed > $timeout)
-            {
-                $response_info = '';
-                $response = '';
-                $timeout = 0;
-            }
-            else
-            {
-                $timeout -= $elapsed;
-            }
-        }
-    }
     $self->_send(Erlang::term_to_binary([
         Erlang::OtpErlangAtom->new('return_sync'), $name, $pattern,
         Erlang::OtpErlangBinary->new($response_info),
@@ -547,7 +465,7 @@ sub timeout_terminate
 sub _null_response
 {
     my $self = shift;
-    my ($command, $name, $pattern, $request_info, $request,
+    my ($request_type, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
     return '';
 }
@@ -557,11 +475,6 @@ sub _callback
     my $self = shift;
     my ($command, $name, $pattern, $request_info, $request,
         $timeout, $priority, $trans_id, $pid) = @_;
-    if ($self->{_request_timeout_adjustment})
-    {
-        $self->{_request_timer} = _milliseconds();
-        $self->{_request_timeout} = $timeout;
-    }
     my $function;
     if (! defined($self->{_callbacks}{$pattern}))
     {
@@ -587,8 +500,13 @@ sub _callback
             {
                 $response = $response_info;
                 $response_info = '';
+                if (! defined($response))
+                {
+                    $response = '';
+                }
             }
-            elsif (ref($response_info) ne '')
+            elsif (ref($response_info) ne '' ||
+                   ! defined($response_info))
             {
                 $response_info = '';
             }
@@ -654,8 +572,13 @@ sub _callback
             {
                 $response = $response_info;
                 $response_info = '';
+                if (! defined($response))
+                {
+                    $response = '';
+                }
             }
-            elsif (ref($response_info) ne '')
+            elsif (ref($response_info) ne '' ||
+                   ! defined($response_info))
             {
                 $response_info = '';
             }
@@ -749,8 +672,12 @@ sub _handle_events
         }
         elsif ($command == MESSAGE_REINIT)
         {
-            $i += $j; $j = 4;
-            ($self->{_process_count}) = unpack('L', substr($data, $i, $j));
+            $i += $j; $j = 4 + 4 + 4 + 1;
+            ($self->{_process_count},
+             $self->{_timeout_async},
+             $self->{_timeout_sync},
+             $self->{_priority_default}) = unpack("L3 c",
+                                                  substr($data, $i, $j));
             $i += $j;
         }
         elsif ($command == MESSAGE_KEEPALIVE)
@@ -840,15 +767,14 @@ sub _poll_request
                 $process_count_max,
                 $process_count_min,
                 $prefix_size) = unpack('L5', substr($data, $i, $j));
-            $i += $j; $j = $prefix_size + 4 + 4 + 4 + 4 + 1 + 1;
+            $i += $j; $j = $prefix_size + 4 + 4 + 4 + 4 + 1;
             my ($prefix,
                 $timeout_initialize,
                 $timeout_async,
                 $timeout_sync,
                 $timeout_terminate,
-                $priority_default,
-                $request_timeout_adjustment) = unpack("Z$prefix_size L4 c C",
-                                                      substr($data, $i, $j));
+                $priority_default) = unpack("Z$prefix_size L4 c",
+                                            substr($data, $i, $j));
             $i += $j;
             if ($i != $data_size)
             {
@@ -859,8 +785,7 @@ sub _poll_request
                     $process_count_max, $process_count_min,
                     $prefix, $timeout_initialize,
                     $timeout_sync, $timeout_async,
-                    $timeout_terminate, $priority_default,
-                    $request_timeout_adjustment);
+                    $timeout_terminate, $priority_default);
         }
         elsif ($command == MESSAGE_SEND_ASYNC ||
                $command == MESSAGE_SEND_SYNC)
@@ -973,12 +898,16 @@ sub _poll_request
         }
         elsif ($command == MESSAGE_REINIT)
         {
-            $i += $j; $j = 4;
-            ($self->{_process_count}) = unpack('L', substr($data, $i, $j));
+            $i += $j; $j = 4 + 4 + 4 + 1;
+            ($self->{_process_count},
+             $self->{_timeout_async},
+             $self->{_timeout_sync},
+             $self->{_priority_default}) = unpack("L3 c",
+                                                  substr($data, $i, $j));
             $i += $j; $j = 4;
             if ($i == $data_size)
             {
-                1;
+                $data = '';
             }
             elsif ($i < $data_size)
             {
@@ -996,7 +925,7 @@ sub _poll_request
             $i += $j; $j = 4;
             if ($i == $data_size)
             {
-                1;
+                $data = '';
             }
             elsif ($i < $data_size)
             {
@@ -1051,7 +980,7 @@ sub _poll_request
         {
             return 1;
         }
-    
+
         $data = $self->_recv($data);
         $data_size = length($data);
         if ($data_size == 0)
@@ -1108,13 +1037,6 @@ sub _text_key_value_parse
     return %result;
 }
 
-sub request_http_qs_parse
-{
-    my $self = shift;
-    my ($request) = @_;
-    return $self->_text_key_value_parse($request);
-}
-
 sub info_key_value_parse
 {
     my $self = shift;
@@ -1136,15 +1058,16 @@ sub _send
 sub _recv
 {
     my $self = shift;
-    my ($data) = @_;
+    my ($data_old) = @_;
+    my $data = '';
     my $fragment = '';
     my $read;
-    my $i = length($data);
     if ($self->{_use_header})
     {
+        my $i = 0;
         while ($i < 4)
         {
-            $read = sysread($self->{_s}, $fragment, $self->{_size});
+            $read = sysread($self->{_s}, $fragment, 4 - $i);
             if (! defined($read) || $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
@@ -1152,12 +1075,13 @@ sub _recv
             $i += $read;
             $data .= $fragment;
         }
-        my ($total) = unpack('N', substr($data, 0, 4));
-        $i -= 4;
-        $data = substr($data, 4);
+        my ($total) = unpack('N', $data);
+        $i = 0;
+        $data = $data_old;
         while ($i < $total)
         {
-            $read = sysread($self->{_s}, $fragment, $self->{_size});
+            $read = sysread($self->{_s}, $fragment,
+                            _min($total - $i, $self->{_size}));
             if (! defined($read) || $read == 0)
             {
                 die CloudI::MessageDecodingException->new();
@@ -1168,6 +1092,8 @@ sub _recv
     }
     else
     {
+        $data = $data_old;
+        my $i = length($data);
         my $ready = 1;
         while ($ready)
         {
@@ -1204,6 +1130,19 @@ sub _milliseconds
     return $seconds * 1000 + $microseconds / 1000;
 }
 
+sub _min
+{
+    my ($min, @args) = @_;
+    for my $v (@args)
+    {
+        if ($v < $min)
+        {
+            $min = $v;
+        }
+    }
+    return $min;
+}
+
 sub _max
 {
     my ($max, @args) = @_;
