@@ -256,10 +256,10 @@ reconnect(#state{protocol = Protocol, host = Host, port = Port,
     end.
 
 prepare_batch_send(Time) ->
-    erlang:send_after(Time, ?MODULE, {cloudi_x_exometer_influxdb, send}).
+    erlang:send_after(Time, self(), {cloudi_x_exometer_influxdb, send}).
 
 prepare_reconnect() ->
-    erlang:send_after(1000, ?MODULE, {cloudi_x_exometer_influxdb, reconnect}).
+    erlang:send_after(1000, self(), {cloudi_x_exometer_influxdb, reconnect}).
 
 -spec maybe_send(list(), list(), map(), map(), state()) ->
     {ok, state()} | {error, term()}.
