@@ -8,49 +8,35 @@
 %%% database agnostic map-reduce.  See the hexpi test for example usage.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2012-2015, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2012-2015 Michael Truog
-%%% @version 1.4.1 {@date} {@time}
+%%% Copyright (c) 2012-2018 Michael Truog <mjtruog at protonmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
+%%%
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2012-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_map_reduce).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -66,15 +52,41 @@
 -define(DEFAULT_MAP_REDUCE_MODULE,     undefined).
 -define(DEFAULT_MAP_REDUCE_ARGUMENTS,         []).
 -define(DEFAULT_CONCURRENCY,                 1.0). % schedulers multiplier
+-define(DEFAULT_LOG_EXECUTION_TIME,         true).
+
+-type map_send_args() :: list().
+-export_type([map_send_args/0]).
 
 -record(state,
     {
-        map_reduce_module,
-        map_reduce_state,
-        map_count,
-        map_requests       % trans_id -> send_args
+        time_start :: cloudi_timestamp:seconds_monotonic(),
+        log_execution_time :: boolean(),
+        map_reduce_module :: module(),
+        map_reduce_state :: any(),
+        map_count :: pos_integer(),
+        map_requests :: #{cloudi_service:trans_id() := map_send_args()}
     }).
 
+-record(init_begin,
+    {
+        service :: pid(),
+        prefix :: string(),
+        timeout :: cloudi_service_api:timeout_initialize_value_milliseconds(),
+        time_start :: cloudi_timestamp:seconds_monotonic(),
+        log_execution_time :: boolean(),
+        map_reduce_module :: module(),
+        map_reduce_args :: list(),
+        concurrency :: number()
+    }).
+
+-record(init_end,
+    {
+        state :: undefined | #state{},
+        error = undefined :: any()
+    }).
+
+-define(INFO_RETRY_INTERVAL, 500). % milliseconds
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -95,16 +107,16 @@
 
 -callback cloudi_service_map_reduce_send(ModuleReduceState :: any(),
                                          Dispatcher :: pid()) ->
-    {'ok', SendArgs :: list(), NewModuleReduceState :: any()} |
+    {'ok', SendArgs :: map_send_args(), NewModuleReduceState :: any()} |
     {'done', NewModuleReduceState :: any()} |
     {'error', Reason :: any()}.
 
--callback cloudi_service_map_reduce_resend(SendArgs :: list(),
+-callback cloudi_service_map_reduce_resend(SendArgs :: map_send_args(),
                                            ModuleReduceState :: any()) ->
-    {'ok', NewSendArgs :: list(), NewModuleReduceState :: any()} |
+    {'ok', NewSendArgs :: map_send_args(), NewModuleReduceState :: any()} |
     {'error', Reason :: any()}.
 
--callback cloudi_service_map_reduce_recv(SendArgs :: list(),
+-callback cloudi_service_map_reduce_recv(SendArgs :: map_send_args(),
                                          ResponseInfo :: any(),
                                          Response :: any(),
                                          Timeout :: non_neg_integer(),
@@ -126,15 +138,19 @@
 %%% Callback functions from cloudi_service
 %%%------------------------------------------------------------------------
 
-cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
+cloudi_service_init(Args, Prefix, Timeout, _Dispatcher) ->
     Defaults = [
         {map_reduce,             ?DEFAULT_MAP_REDUCE_MODULE},
         {map_reduce_args,        ?DEFAULT_MAP_REDUCE_ARGUMENTS},
-        {concurrency,            ?DEFAULT_CONCURRENCY}],
-    [MapReduceModule, MapReduceArguments, Concurrency] =
+        {concurrency,            ?DEFAULT_CONCURRENCY},
+        {log_execution_time,     ?DEFAULT_LOG_EXECUTION_TIME}],
+    [MapReduceModule, MapReduceArgs, Concurrency, LogExecutionTime] =
         cloudi_proplists:take_values(Defaults, Args),
-    true = is_atom(MapReduceModule) and (MapReduceModule /= undefined),
-    true = is_list(MapReduceArguments),
+    TimeStart = cloudi_timestamp:seconds_monotonic(),
+    true = is_atom(MapReduceModule) andalso (MapReduceModule /= undefined),
+    true = is_list(MapReduceArgs),
+    true = is_number(Concurrency) andalso (Concurrency > 0),
+    true = is_boolean(LogExecutionTime),
     case application:load(MapReduceModule) of
         ok ->
             ok = cloudi_x_reltool_util:application_start(MapReduceModule,
@@ -145,9 +161,16 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         {error, _} ->
             ok = cloudi_x_reltool_util:module_loaded(MapReduceModule)
     end,
-    cloudi_service:self(Dispatcher) !
-        {init, Prefix, Timeout,
-         MapReduceModule, MapReduceArguments, Concurrency},
+    % cloudi_service_init/4 is always executed by the service process
+    Service = self(),
+    Service !  #init_begin{service = Service,
+                           prefix = Prefix,
+                           timeout = Timeout,
+                           time_start = TimeStart,
+                           log_execution_time = LogExecutionTime,
+                           map_reduce_module = MapReduceModule,
+                           map_reduce_args = MapReduceArgs,
+                           concurrency = Concurrency},
     {ok, undefined}.
 
 cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
@@ -155,51 +178,60 @@ cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
                               State, _Dispatcher) ->
     {reply, <<>>, State}.
 
-cloudi_service_handle_info({init, Prefix, Timeout,
-                            MapReduceModule, MapReduceArguments, Concurrency},
+cloudi_service_handle_info(#init_begin{service = Service} = InitBegin,
                            undefined, Dispatcher) ->
-    % cloudi_service_map_reduce_new/3 execution occurs outside of
-    % cloudi_service_init/3 to allow send_sync and recv_sync function calls
+    % cloudi_service_map_reduce_new/5 execution occurs outside of
+    % cloudi_service_init/4 to allow send_sync and recv_async function calls
     % because no Erlang process linking/spawning/etc. should be occurring,
-    % only algorithmic initialization
-    MapCount = cloudi_concurrency:count(Concurrency),
-    case MapReduceModule:cloudi_service_map_reduce_new(MapReduceArguments,
-                                                       MapCount,
-                                                       Prefix,
-                                                       Timeout,
-                                                       Dispatcher) of
-        {ok, MapReduceState} ->
-            case map_send(MapCount, dict:new(), Dispatcher,
-                          MapReduceModule, MapReduceState) of
-                {ok, MapRequests, NewMapReduceState} ->
-                    {noreply, #state{map_reduce_module = MapReduceModule,
-                                     map_reduce_state = NewMapReduceState,
-                                     map_count = MapCount,
-                                     map_requests = MapRequests}};
-                {error, _} = Error ->
-                    {stop, Error, undefined}
-            end;
-        {error, _} = Error ->
-            {stop, Error, undefined}
+    % only algorithmic initialization.  Initialization is done in a temporary
+    % process so there is no blocking problem (with send_sync or recv_async)
+    % if duo_mode == true.  No timeout is enforced on
+    % cloudi_service_map_reduce_new/5 execution.
+    _ = erlang:spawn_link(fun() ->
+        case init(InitBegin, Dispatcher) of
+            {noreply, State} ->
+                Service ! #init_end{state = State};
+            {stop, Reason, State} when Reason /= undefined ->
+                Service ! #init_end{state = State,
+                                    error = Reason}
+        end,
+        true = erlang:unlink(Service)
+    end),
+    {noreply, undefined};
+
+cloudi_service_handle_info(#init_end{state = State,
+                                     error = Error},
+                           undefined, _Dispatcher) ->
+    if
+        Error =:= undefined ->
+            {noreply, State};
+        true ->
+            {stop, Error, State}
     end;
 
+cloudi_service_handle_info(Request, undefined, Dispatcher) ->
+    % waiting for #init_end{} still
+    erlang:send_after(?INFO_RETRY_INTERVAL,
+                      cloudi_service:self(Dispatcher), Request),
+    {noreply, undefined};
+
 cloudi_service_handle_info(#timeout_async_active{trans_id = TransId} = Request,
                            #state{map_reduce_module = MapReduceModule,
                                   map_reduce_state = MapReduceState,
                                   map_requests = MapRequests} = State,
                            Dispatcher) ->
-    case dict:find(TransId, MapRequests) of
+    case maps:find(TransId, MapRequests) of
         {ok, [_ | SendArgs]} ->
-            NextMapRequests = dict:erase(TransId, MapRequests),
+            NextMapRequests = maps:remove(TransId, MapRequests),
             case MapReduceModule:cloudi_service_map_reduce_resend(
                 [Dispatcher | SendArgs], MapReduceState) of
                 {ok, NewSendArgs, NewMapReduceState} ->
                     case erlang:apply(cloudi_service, send_async_active,
                                       NewSendArgs) of
                         {ok, NewTransId} ->
-                            NewMapRequests = dict:store(NewTransId,
-                                                        NewSendArgs,
-                                                        NextMapRequests),
+                            NewMapRequests = maps:put(NewTransId,
+                                                      NewSendArgs,
+                                                      NextMapRequests),
                             {noreply,
                              State#state{map_reduce_state = NewMapReduceState,
                                          map_requests = NewMapRequests}};
@@ -221,13 +253,13 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                   map_reduce_state = MapReduceState,
                                   map_requests = MapRequests} = State,
                            Dispatcher) ->
-    case dict:find(TransId, MapRequests) of
+    case maps:find(TransId, MapRequests) of
         {ok, [_ | SendArgs]} ->
             case MapReduceModule:cloudi_service_map_reduce_recv(
                 [Dispatcher | SendArgs], ResponseInfo, Response,
                 Timeout, TransId, MapReduceState, Dispatcher) of
                 {ok, NextMapReduceState} ->
-                    case map_send(dict:erase(TransId, MapRequests),
+                    case map_send(maps:remove(TransId, MapRequests),
                                   Dispatcher, MapReduceModule,
                                   NextMapReduceState) of
                         {ok, NewMapRequests, NewMapReduceState} ->
@@ -238,10 +270,10 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                             {stop, Error, State}
                     end;
                 {done, NewMapReduceState} ->
-                    NewMapRequests = dict:erase(TransId, MapRequests),
+                    NewMapRequests = maps:remove(TransId, MapRequests),
                     NewState = State#state{map_reduce_state = NewMapReduceState,
                                            map_requests = NewMapRequests},
-                    case dict:size(NewMapRequests) of
+                    case maps:size(NewMapRequests) of
                         0 ->
                             {stop, shutdown, NewState};
                         _ ->
@@ -259,6 +291,14 @@ cloudi_service_handle_info(Request, State, Dispatcher) ->
 
 cloudi_service_terminate(_Reason, _Timeout, undefined) ->
     ok;
+cloudi_service_terminate(shutdown, _Timeout,
+                         #state{time_start = TimeStart,
+                                log_execution_time = true,
+                                map_requests = #{}}) ->
+    TimeEnd = cloudi_timestamp:seconds_monotonic(),
+    ?LOG_INFO("total time taken was ~p hours",
+              [hours_elapsed(TimeEnd, TimeStart)]),
+    ok;
 cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
     ok.
 
@@ -266,6 +306,37 @@ cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+init(#init_begin{prefix = Prefix,
+                 timeout = Timeout,
+                 time_start = TimeStart,
+                 log_execution_time = LogExecutionTime,
+                 map_reduce_module = MapReduceModule,
+                 map_reduce_args = MapReduceArgs,
+                 concurrency = Concurrency},
+     Dispatcher) ->
+    MapCount = cloudi_concurrency:count(Concurrency),
+    case MapReduceModule:cloudi_service_map_reduce_new(MapReduceArgs,
+                                                       MapCount,
+                                                       Prefix,
+                                                       Timeout,
+                                                       Dispatcher) of
+        {ok, MapReduceState} ->
+            case map_send(MapCount, #{}, Dispatcher,
+                          MapReduceModule, MapReduceState) of
+                {ok, MapRequests, NewMapReduceState} ->
+                    {noreply, #state{time_start = TimeStart,
+                                     log_execution_time = LogExecutionTime,
+                                     map_reduce_module = MapReduceModule,
+                                     map_reduce_state = NewMapReduceState,
+                                     map_count = MapCount,
+                                     map_requests = MapRequests}};
+                {error, _} = Error ->
+                    {stop, Error, undefined}
+            end;
+        {error, _} = Error ->
+            {stop, Error, undefined}
+    end.
+
 map_send(MapRequests, Dispatcher, MapReduceModule, MapReduceState) ->
     map_send(1, MapRequests, Dispatcher, MapReduceModule, MapReduceState).
 
@@ -279,7 +350,7 @@ map_send(Count, MapRequests, Dispatcher, MapReduceModule, MapReduceState) ->
             case erlang:apply(cloudi_service, send_async_active, SendArgs) of
                 {ok, TransId} ->
                     map_send(Count - 1,
-                             dict:store(TransId, SendArgs, MapRequests),
+                             maps:put(TransId, SendArgs, MapRequests),
                              Dispatcher, MapReduceModule, NewMapReduceState);
                 {error, _} = Error ->
                     Error
@@ -302,7 +373,7 @@ cloudi_service_map_reduce_info(Request,
             {noreply, State#state{map_reduce_state = NewMapReduceState}};
         {done, NewMapReduceState} ->
             NewState = State#state{map_reduce_state = NewMapReduceState},
-            case dict:size(MapRequests) of
+            case maps:size(MapRequests) of
                 0 ->
                     {stop, shutdown, NewState};
                 _ ->
@@ -312,3 +383,7 @@ cloudi_service_map_reduce_info(Request,
             {stop, Error, State}
     end.
 
+hours_elapsed(Seconds1, Seconds0)
+    when Seconds1 >= Seconds0 ->
+    erlang:round(((Seconds1 - Seconds0) / (60 * 60)) * 10) / 10.
+
